@@ -4,10 +4,13 @@
 #include "Filter.h"
 #include "Image.h"
 
+#include "nvthread/ParallelFor.h"
+
 #include "nvmath/Color.h"
 #include "nvmath/Vector.inl"
 #include "nvmath/Matrix.inl"
 #include "nvmath/ftoi.h"
+#include "nvmath/Gamma.h"
 
 #include "nvcore/Utils.h" // max
 #include "nvcore/Ptr.h"
@@ -27,6 +30,13 @@ FloatImage::FloatImage() : m_componentCount(0), m_width(0), m_height(0), m_depth
 {
 }
 
+FloatImage::FloatImage(const FloatImage & img) : m_componentCount(0), m_width(0), m_height(0), m_depth(0),
+    m_pixelCount(0), m_floatCount(0), m_mem(NULL)
+{
+    allocate(img.m_componentCount, img.m_width, img.m_height, img.m_depth);
+    memcpy(m_mem, img.m_mem, m_floatCount * sizeof(float));
+}
+
 /// Ctor. Init from image.
 FloatImage::FloatImage(const Image * img) : m_componentCount(0), m_width(0), m_height(0), m_depth(0),
     m_pixelCount(0), m_floatCount(0), m_mem(NULL)
@@ -40,27 +50,32 @@ FloatImage::~FloatImage()
     free();
 }
 
-
 /// Init the floating point image from a regular image.
 void FloatImage::initFrom(const Image * img)
 {
     nvCheck(img != NULL);
 
-    allocate(4, img->width(), img->height(), img->depth());
+    uint channel_count = 3;
+    if (img->format == Image::Format_ARGB) channel_count = 4;
+
+    allocate(channel_count, img->width, img->height, img->depth);
 
     float * red_channel = channel(0);
     float * green_channel = channel(1);
     float * blue_channel = channel(2);
-    float * alpha_channel = channel(3);
+    float * alpha_channel = (channel_count == 4) ? channel(3) : NULL;
+
+    float scale = 1.0f / 255.0f;
 
     const uint count = m_pixelCount;
     for (uint i = 0; i < count; i++) {
+    //parallel_for(count, 128, [&](int i) {
         Color32 pixel = img->pixel(i);
-        red_channel[i] = float(pixel.r) / 255.0f;
-        green_channel[i] = float(pixel.g) / 255.0f;
-        blue_channel[i] = float(pixel.b) / 255.0f;
-        alpha_channel[i] = float(pixel.a) / 255.0f;
-    }
+        red_channel[i] = float(pixel.r) * scale;
+        green_channel[i] = float(pixel.g) * scale;
+        blue_channel[i] = float(pixel.b) * scale;
+        if (channel_count == 4) alpha_channel[i] = float(pixel.a) * scale;
+    }//);
 }
 
 /// Convert the floating point image to a regular image.
@@ -243,13 +258,29 @@ void FloatImage::clamp(uint baseComponent, uint num, float low, float high)
 /// From gamma to linear space.
 void FloatImage::toLinear(uint baseComponent, uint num, float gamma /*= 2.2f*/)
 {
-    exponentiate(baseComponent, num, gamma);
+    if (gamma == 2.2f) {
+        for (uint c = 0; c < num; c++) {
+            float * ptr = this->channel(baseComponent + c);
+
+            powf_11_5(ptr, ptr, m_pixelCount);
+        }
+    } else {
+        exponentiate(baseComponent, num, gamma);
+    }
 }
 
 /// From linear to gamma space.
 void FloatImage::toGamma(uint baseComponent, uint num, float gamma /*= 2.2f*/)
 {
-    exponentiate(baseComponent, num, 1.0f/gamma);
+    if (gamma == 2.2f) {
+        for (uint c = 0; c < num; c++) {
+            float * ptr = this->channel(baseComponent + c);
+
+            powf_5_11(ptr, ptr, m_pixelCount);
+        }
+    } else {
+        exponentiate(baseComponent, num, 1.0f/gamma);
+    }
 }
 
 /// Exponentiate the elements of the image.
@@ -458,13 +489,17 @@ float FloatImage::sampleLinearClamp(uint c, float x, float y, float z) const
     const float fracY = frac(y);
     const float fracZ = frac(z);
 
+    //x -= fracX;
+    //y -= fracY;
+    //z -= fracZ;
+
     // @@ Using floor in some places, but round in others?
     const int ix0 = ::clamp(ifloor(x), 0, w-1);
     const int iy0 = ::clamp(ifloor(y), 0, h-1);
-    const int iz0 = ::clamp(ifloor(z), 0, h-1);
+    const int iz0 = ::clamp(ifloor(z), 0, d-1);
     const int ix1 = ::clamp(ifloor(x)+1, 0, w-1);
     const int iy1 = ::clamp(ifloor(y)+1, 0, h-1);
-    const int iz1 = ::clamp(ifloor(z)+1, 0, h-1);
+    const int iz1 = ::clamp(ifloor(z)+1, 0, d-1);
 
     return trilerp(c, ix0, iy0, iz0, ix1, iy1, iz1, fracX, fracY, fracZ);
 }
@@ -740,8 +775,8 @@ FloatImage * FloatImage::resize(const Filter & filter, uint w, uint h, WrapMode
         dst_image->allocate(m_componentCount, w, h);
 
         // @@ We could avoid this allocation, write directly to dst_plane.
-        Array<float> tmp_column(h);
-        tmp_column.resize(h);
+        //Array<float> tmp_column(h);
+        //tmp_column.resize(h);
 
         for (uint c = 0; c < m_componentCount; c++)
         {
@@ -750,19 +785,21 @@ FloatImage * FloatImage::resize(const Filter & filter, uint w, uint h, WrapMode
                 float * tmp_plane = tmp_image->plane(c, z);
 
                 for (uint y = 0; y < m_height; y++) {
+                //parallel_for(m_height, [&](int y) {
                     this->applyKernelX(xkernel, y, z, c, wm, tmp_plane + y * w);
-                }
+                }//);
 
                 float * dst_plane = dst_image->plane(c, z);
 
                 for (uint x = 0; x < w; x++) {
-                    tmp_image->applyKernelY(ykernel, x, z, c, wm, tmp_column.buffer());
+                //parallel_for(w, [&](int x) {
+                    tmp_image->applyKernelY(ykernel, x, z, c, wm, dst_plane + x, w);
 
                     // @@ We could avoid this copy, write directly to dst_plane.
-                    for (uint y = 0; y < h; y++) {
+                    /*for (uint y = 0; y < h; y++) {
                         dst_plane[y * w + x] = tmp_column[y];
-                    }
-                }
+                    }*/
+                }//);
             }
         }
     }
@@ -823,7 +860,7 @@ FloatImage * FloatImage::resize(const Filter & filter, uint w, uint h, uint d, W
 
         for (uint z = 0; z < d; z++ ) {
             for (uint x = 0; x < w; x++) {
-                tmp_image2->applyKernelY(ykernel, x, z, c, wm, tmp_column.buffer());
+                tmp_image2->applyKernelY(ykernel, x, z, c, wm, tmp_column.buffer(), 1);
 
                 for (uint y = 0; y < h; y++) {
                     dst_channel[z * h * w + y * w + x] = tmp_column[y];
@@ -873,7 +910,7 @@ FloatImage * FloatImage::resize(const Filter & filter, uint w, uint h, WrapMode
                 float * dst_plane = dst_image->plane(c, z);
 
                 for (uint x = 0; x < w; x++) {
-                    tmp_image->applyKernelY(ykernel, x, z, c, wm, tmp_column.buffer());
+                    tmp_image->applyKernelY(ykernel, x, z, c, wm, tmp_column.buffer(), 1);
 
                     // @@ Avoid this copy, write directly to dst_plane.
                     for (uint y = 0; y < h; y++) {
@@ -944,7 +981,7 @@ FloatImage * FloatImage::resize(const Filter & filter, uint w, uint h, uint d, W
 
         for (uint z = 0; z < d; z++ ) {
             for (uint x = 0; x < w; x++) {
-                tmp_image2->applyKernelY(ykernel, x, z, c, wm, tmp_column.buffer());
+                tmp_image2->applyKernelY(ykernel, x, z, c, wm, tmp_column.buffer(), 1);
 
                 for (uint y = 0; y < h; y++) {
                     dst_channel[z * h * w + y * w + x] = tmp_column[y];
@@ -1107,7 +1144,7 @@ void FloatImage::applyKernelX(const PolyphaseKernel & k, int y, int z, uint c, W
 }
 
 /// Apply 1D vertical kernel at the given coordinates and return result.
-void FloatImage::applyKernelY(const PolyphaseKernel & k, int x, int z, uint c, WrapMode wm, float * __restrict output) const
+void FloatImage::applyKernelY(const PolyphaseKernel & k, int x, int z, uint c, WrapMode wm, float * __restrict output, int output_stride) const
 {
     const uint length = k.length();
     const float scale = float(length) / float(m_height);
@@ -1134,7 +1171,7 @@ void FloatImage::applyKernelY(const PolyphaseKernel & k, int x, int z, uint c, W
             sum += k.valueAt(i, j) * channel[idx];
         }
 
-        output[i] = sum;
+        output[i * output_stride] = sum;
     }
 }
 
@@ -1208,7 +1245,7 @@ void FloatImage::applyKernelX(const PolyphaseKernel & k, int y, int z, uint c, u
 }
 
 /// Apply 1D vertical kernel at the given coordinates and return result.
-void FloatImage::applyKernelY(const PolyphaseKernel & k, int x, int z, uint c, uint a, WrapMode wm, float * __restrict output) const
+void FloatImage::applyKernelY(const PolyphaseKernel & k, int x, int z, uint c, uint a, WrapMode wm, float * __restrict output, int output_stride) const
 {
     const uint length = k.length();
     const float scale = float(length) / float(m_height);
@@ -1239,7 +1276,7 @@ void FloatImage::applyKernelY(const PolyphaseKernel & k, int x, int z, uint c, u
             sum += w * channel[idx];
         }
 
-        output[i] = sum / norm;
+        output[i * output_stride] = sum / norm;
     }
 }
 
@@ -1356,7 +1393,7 @@ float FloatImage::alphaTestCoverage(float alphaRef, int alphaChannel, float alph
     
     return coverage / float(w * h);
 #else
-    const uint n = 8;
+    const uint n = 4;
 
     // If we want subsampling:
     for (uint y = 0; y < h-1; y++) {
@@ -1367,16 +1404,20 @@ float FloatImage::alphaTestCoverage(float alphaRef, int alphaChannel, float alph
             float alpha01 = nv::saturate(pixel(alphaChannel, x+0, y+1, 0) * alphaScale);
             float alpha11 = nv::saturate(pixel(alphaChannel, x+1, y+1, 0) * alphaScale);
 
-            for (float fy = 0.5f/n; fy < 1.0f; fy++) {
-                for (float fx = 0.5f/n; fx < 1.0f; fx++) {
+            float texel_coverage = 0.0f;
+            for (uint sy = 0; sy < n; sy++) {
+                float fy = (sy + 0.5f) / n;
+                for (uint sx = 0; sx < n; sx++) {
+                    float fx = (sx + 0.5f) / n;
                     float alpha = alpha00 * (1 - fx) * (1 - fy) + alpha10 * fx * (1 - fy) + alpha01 * (1 - fx) * fy + alpha11 * fx * fy;
-                    if (alpha > alphaRef) coverage += 1.0f;
+                    if (alpha > alphaRef) texel_coverage += 1.0f;
                 }
             }
+            coverage += texel_coverage / (n * n);
         }
     }
 
-    return coverage / float(w * h * n * n);
+    return coverage / float((w - 1) * (h - 1));
 #endif
 }
 
@@ -1413,11 +1454,19 @@ void FloatImage::scaleAlphaToCoverage(float desiredCoverage, float alphaRef, int
     float minAlphaScale = 0.0f;
     float maxAlphaScale = 4.0f;
     float alphaScale = 1.0f;
+    float bestAlphaScale = 1.0f;
+    float bestError = NV_FLOAT_MAX;
 
-    // Determine desired scale using a binary search. Hardcoded to 8 steps max.
+    // Determine desired scale using a binary search. Hardcoded to 10 steps max.
     for (int i = 0; i < 10; i++) {
         float currentCoverage = alphaTestCoverage(alphaRef, alphaChannel, alphaScale);
 
+        float error = fabsf(currentCoverage - desiredCoverage);
+        if (error < bestError) {
+            bestError = error;
+            bestAlphaScale = alphaScale;
+        }
+
         if (currentCoverage < desiredCoverage) {
             minAlphaScale = alphaScale;
         }
@@ -1432,7 +1481,7 @@ void FloatImage::scaleAlphaToCoverage(float desiredCoverage, float alphaRef, int
     }
 
     // Scale alpha channel.
-    scaleBias(alphaChannel, 1, alphaScale, 0.0f);
+    scaleBias(alphaChannel, 1, bestAlphaScale, 0.0f);
     clamp(alphaChannel, 1, 0.0f, 1.0f); 
 #endif
 #if _DEBUG
