@@ -1,21 +1,14 @@
 // This code is in the public domain -- Ignacio Castao <castano@gmail.com>
 
 #pragma once
-#ifndef NV_MATH_PLANE_H
-#define NV_MATH_PLANE_H
 
-#include "nvmath.h"
 #include "Vector.h"
 
-#if NV_USE_ALTIVEC
-#undef vector
-#endif
-
 namespace nv
 {
     class Matrix;
 
-    class NVMATH_CLASS Plane
+    class Plane
     {
     public:
         Plane();
@@ -29,6 +22,7 @@ namespace nv
 
         Vector3 vector() const;
         float offset() const;
+        Vector3 normal() const;
 
         void operator*=(float s);
 
@@ -41,5 +35,3 @@ namespace nv
 
 
 } // nv namespace
-
-#endif // NV_MATH_PLANE_H
