@@ -39,7 +39,7 @@ using namespace nvtt;
 // Solid angle of an axis aligned quad from (0,0,1) to (x,y,1)
 // See: http://www.fizzmoll11.com/thesis/ for a derivation of this formula.
 static float areaElement(float x, float y) {
-    return atan2(x*y, sqrtf(x*x + y*y + 1));
+    return atan2f(x*y, sqrtf(x*x + y*y + 1));
 }
 
 // Solid angle of a hemicube texel.
@@ -206,19 +206,19 @@ static const Vector3 faceV[6] = {
 
 static Vector2 toPolar(Vector3::Arg v) {
     Vector2 p;
-    p.x = atan2(v.x, v.y);  // theta
+    p.x = atan2f(v.x, v.y);  // theta
     p.y = acosf(v.z);       // phi
     return p;
 }
 
 static Vector2 toPlane(float theta, float phi) {
-    float x = sin(phi) * cos(theta);
-    float y = sin(phi) * sin(theta);
-    float z = cos(phi);
+    float x = sinf(phi) * cosf(theta);
+    float y = sinf(phi) * sinf(theta);
+    float z = cosf(phi);
 
     Vector2 p;
-    p.x = x / fabs(z);
-    p.y = y / fabs(z);
+    p.x = x / fabsf(z);
+    p.y = y / fabsf(z);
     //p.x = tan(phi) * cos(theta);
     //p.y = tan(phi) * sin(theta);
 
@@ -227,8 +227,8 @@ static Vector2 toPlane(float theta, float phi) {
 
 static Vector2 toPlane(Vector3::Arg v) {
     Vector2 p;
-    p.x = v.x / fabs(v.z);
-    p.y = v.y / fabs(v.z);
+    p.x = v.x / fabsf(v.z);
+    p.y = v.y / fabsf(v.z);
     return p;
 }
 
@@ -303,7 +303,10 @@ const Surface & CubeSurface::face(int f) const
 bool CubeSurface::load(const char * fileName, int mipmap)
 {
     if (strEqual(Path::extension(fileName), ".dds")) {
-        nv::DirectDrawSurface dds(fileName);
+        nv::DirectDrawSurface dds;
+        if (!dds.load(fileName)) {
+            return false;
+        }
 
         if (!dds.isValid()/* || !dds.isSupported()*/) {
             return false;
@@ -529,6 +532,24 @@ void CubeSurface::clamp(int channel, float low/*= 0.0f*/, float high/*= 1.0f*/)
 
 CubeSurface CubeSurface::irradianceFilter(int size, EdgeFixup fixupMethod) const
 {
+    // Evaluate spherical harmonic for each output texel.
+    CubeSurface output;
+    output.m->allocate(size);
+
+    Sh2 shr, shg, shb;
+
+    computeIrradianceSH3(0, shr.coef);
+    computeIrradianceSH3(1, shg.coef);
+    computeIrradianceSH3(2, shb.coef);
+
+    // @@ Sample spherical harmonic from every direction.
+
+    return CubeSurface();
+}
+
+
+void CubeSurface::computeLuminanceIrradianceSH3(float coef[9]) const{
+
     m->allocateTexelTable();
 
     // Transform this cube to spherical harmonic basis
@@ -537,6 +558,10 @@ CubeSurface CubeSurface::irradianceFilter(int size, EdgeFixup fixupMethod) const
     // For each texel of the input cube.
     const uint edgeLength = m->edgeLength;
     for (uint f = 0; f < 6; f++) {
+
+        const Surface & inputFace = m->face[f];
+        const FloatImage * inputImage = inputFace.m->image;
+
         for (uint y = 0; y < edgeLength; y++) {
             for (uint x = 0; x < edgeLength; x++) {
 
@@ -546,30 +571,62 @@ CubeSurface CubeSurface::irradianceFilter(int size, EdgeFixup fixupMethod) const
                 Sh2 shDir;
                 shDir.eval(dir);
 
-                sh.addScaled(sh, solidAngle);
+                float r = inputImage->pixel(0, x, y, 0);
+                float g = inputImage->pixel(1, x, y, 0);
+                float b = inputImage->pixel(2, x, y, 0);
+                float lum = 0.333f * (r + g + b);  // @@ use the proper luminance formula.
+
+                sh.addScaled(shDir, lum * solidAngle);
             }
         }
     }
 
-
-    // Evaluate spherical harmonic for each output texel.
-    CubeSurface output;
-    output.m->allocate(size);
-
-
-
-
-    // @@ TODO
-    return CubeSurface();
+    for (int i = 0; i < 9; i++) {
+        coef[i] = sh.coef[i];
+    }
 }
 
 
+void CubeSurface::computeIrradianceSH3(int channel, float coef[9]) const {
+
+    m->allocateTexelTable();
+
+    // Transform this cube to spherical harmonic basis
+    Sh2 sh;
+
+    // For each texel of the input cube.
+    const uint edgeLength = m->edgeLength;
+    for (uint f = 0; f < 6; f++) {
+
+        const Surface & inputFace = m->face[f];
+        const FloatImage * inputImage = inputFace.m->image;
+
+        for (uint y = 0; y < edgeLength; y++) {
+            for (uint x = 0; x < edgeLength; x++) {
+
+                Vector3 dir = m->texelTable->direction(f, x, y);
+                float solidAngle = m->texelTable->solidAngle(f, x, y);
+
+                Sh2 shDir;
+                shDir.eval(dir);
+
+                float c = inputImage->pixel(channel, x, y, 0);
+
+                sh.addScaled(shDir, c * solidAngle);
+            }
+        }
+    }
+
+    for (int i = 0; i < 9; i++) {
+        coef[i] = sh.elemAt(i);
+    }
+}
 
 
 // Convolve filter against this cube.
 Vector3 CubeSurface::Private::applyAngularFilter(const Vector3 & filterDir, float coneAngle, float * filterTable, int tableSize)
 {
-    const float cosineConeAngle = cos(coneAngle);
+    const float cosineConeAngle = cosf(coneAngle);
     nvDebugCheck(cosineConeAngle >= 0);
 
     Vector3 color(0);
@@ -690,7 +747,7 @@ Vector3 CubeSurface::Private::applyAngularFilter(const Vector3 & filterDir, floa
 // Convolve filter against this cube.
 Vector3 CubeSurface::Private::applyCosinePowerFilter(const Vector3 & filterDir, float coneAngle, float cosinePower)
 {
-    const float cosineConeAngle = cos(coneAngle);
+    const float cosineConeAngle = cosf(coneAngle);
     nvDebugCheck(cosineConeAngle >= 0);
 
     Vector3 color(0);
@@ -790,6 +847,8 @@ Vector3 CubeSurface::Private::applyCosinePowerFilter(const Vector3 & filterDir,
     return color;
 }
 
+
+
 #include "nvthread/ParallelFor.h"
 
 struct ApplyAngularFilterContext {
@@ -832,7 +891,7 @@ CubeSurface CubeSurface::cosinePowerFilter(int size, float cosinePower, EdgeFixu
     CubeSurface filteredCube;
     filteredCube.m->allocate(size);
 
-    // Texel table is stored along with the surface so that it's compute only once.
+    // Texel table is stored along with the surface so that it's computed only once.
     m->allocateTexelTable();
 
     const float threshold = 0.001f;
@@ -968,7 +1027,6 @@ CubeSurface CubeSurface::fastResample(int size, EdgeFixup fixupMethod) const
     return resampledCube;
 }
 
-
 void CubeSurface::toLinear(float gamma)
 {
     if (isNull()) return;
